#!/usr/bin/env python3
"""
Script for dumping workers presence from HR.
"""
import os
import logging
import datetime
import argparse

from HRlo import HRpresence
from HRlo import HRauth
from HRlo import HRget

from .scripts_utils import logit

defaults = {
    'log_file'   : None,
    #'log_file'   : 'dump_presents.log',
    'csv_prefix' : 'workers_list',
    'csv_dir'    : os.getcwd(),
    'verbose'    : 0,
}


def dump_presents(**kwargs):

    config = {}
    config.update(defaults)
    config.update(kwargs)

    log = logit(**config)

    time_format = datetime.datetime.now().strftime("%Y%m%d-%H%M%S")

    # make csv dir if not exists
    if not os.path.exists(config['csv_dir']):
        os.makedirs(config['csv_dir'])

    csv_file = '{}_{}.csv'.format(config['csv_prefix'], time_format)
    csv_file = os.path.join(config['csv_dir'], csv_file)

    log.log("getting list of workers")

    hr_auth = HRauth.HRauth()
    hr_get  = HRget.HRget(hr_auth, verbose=False)
    workers = HRpresence.HRpresence(hr_get.presence())

    workers.dump_csv(csv_file)

    log.log("dump data to file: {}".format(csv_file))
    log.log("num workers found {}".format(len(workers.presence)))


def add_parser(parser):

    _parser_dump_presents = parser.add_argument_group('presents options')

    _parser_dump_presents.add_argument('-v', '--verbose',
                                       action='count',
                                       default=0,
                                       help='increase verbosity')

    _parser_dump_presents.add_argument('--csv-prefix',
                                       default=defaults['csv_prefix'],
                                       help='name prefix for csv file. (default %(default)s)')

    _parser_dump_presents.add_argument('--csv-dir',
                                       default=defaults['csv_dir'],
                                       help='directory of csv file. (default %(default)s)')

    _parser_dump_presents.add_argument('--log-file',
                                       default=defaults['log_file'],
                                       help='log file. (default %(default)s)')



def main():

    parser = argparse.ArgumentParser(prog='dump presents',
                                     description=__doc__,
                                     formatter_class=argparse.RawTextHelpFormatter)

    add_parser(parser)

    args = parser.parse_args()

    dump_presents(**vars(args))


if __name__ == '__main__':
    main()




#!/usr/bin/env python3
"""
Script for report from HR presence.
"""
import os
import re
import glob
import argparse
import configparser
from collections import OrderedDict

from HRlo import HRpresence

from .scripts_utils import logit
from .dump_presents import defaults as dump_presents_defaults


defaults = {
    'log_file'   : None,
    #'log_file'   : 'report.log',
    'csv_prefix' : dump_presents_defaults['csv_prefix'],
    'csv_dir'    : dump_presents_defaults['csv_dir'],
    'config_file' : os.path.join(os.getcwd(), 'report.ini'),
    'verbose'    : 0,
}

def get_values_from_config(config_file, section):
    cfg = configparser.ConfigParser()
    cfg.read(config_file)
    d = {}
    if section in cfg.sections():
        # get value from option
        for key in cfg.options(section):
            # expect quota for values
            d[key] = re.findall('"(.+?)"', cfg.get(section, key))
    return d


def get_list_from_file(list_files):
    l = []
    for f in list_files:
        if os.path.isfile(f):
            with open(f, 'r') as fp:
                name_list = fp.readlines()
            l.extend([ n.strip() for n in name_list if '#' not in n ])
    return l


def include(l, config=None):
    # get dict with key and list of values
    dinc = get_values_from_config(config, 'include')

    # read list from file
    ls_file = []
    if dinc.get('file'):
        for n in get_list_from_file(dinc['file']):
            list_in = [ w for w in l if w.is_like('name', "^" + n) ]
            if any(list_in):
                worker = list_in[0]
                if not any([ True for w in ls_file if worker['name'] == w['name'] ]):
                    ls_file.append(worker)
    else:
        ls_file = l

    # include workers with key==value from list
    ls = []
    for key, values in dinc.items():
        if (key=='file'):
            continue
        for v in values:
            for worker in [ w for w in ls_file if v in w[key] ]:
                if not any([ True for w in ls if worker['name'] == w['name'] ]):
                    ls.append(worker)
    else:
        ls = ls_file

    return ls


def exclude(l, config=None):
    # get dict with key and list of values
    exclude = get_values_from_config(config, 'exclude')
    # exclude workers with key==value from list
    for key, values in exclude.items():
        for v in values:
            l = [ w for w in l if v not in w[key] ]
    return l


def remove(l, config=None):
    # get dict with key and list of values
    remove = get_values_from_config(config, 'remove')
    # remove workers with key==value from list
    for key, values in remove.items():
        for v in values:
            l = [ w for w in l if v not in w[key] ]
    return l


def monitor(l, m, config=None):
    for key, values in get_values_from_config(config, 'monitor').items():
        for v in values:
            if not m.get(v): m[v] = set()
            for w in l:
                if v in w[key]:
                    #if not m.get(v): m[v] = set()
                    m[v].add(w['name'])
    return m


def refine(l, config=None):
    d = {}
    for key, values in get_values_from_config(config, 'refine').items():
        d[key] = {}
        for v in values:
            d[key][v] = [ w for w in l if v in w[key] ]
    return d


def allowed_list(config=None):
    l = []
    for key, values in get_values_from_config(config, 'allowed').items():
        # names of workers
        if key == 'name':
            l.extend(values)
        # files with names of workers
        if key == 'file':
            for f in values:
                if os.path.isfile(f):
                    with open(f, 'r') as fp:
                        name_list = fp.readlines()
                    l.extend([ n.strip() for n in name_list if '#' not in n ])
    return l


def stats_str(l, tot):
    _percentage = 0.0
    if len(tot):
        _percentage = 100.0*len(l)/len(tot)
    return "{:3d} / {:3d} = {:.1f}%".format(len(l), len(tot), _percentage)


def report(**kwargs):

    config = {}
    config.update(defaults)
    config.update(kwargs)

    log = logit(**config)

    csv_file = os.path.join(config['csv_dir'], '{}*.csv'.format(config['csv_prefix']))

    list_files = glob.glob(csv_file)

    if not list_files:
        return

    allowed = allowed_list(config.get('config_file'))

    presents_name = set()
    presents_list = []

    dmonitor = {}

    for f in list_files:

        dcsv = HRpresence.HRpresence.read_csv(f)

        p = HRpresence.HRpresence(dcsv)

        # total worker list used for percentage
        list_total = p.presence
        list_total = remove(list_total, config.get('config_file'))
        list_total = include(list_total, config.get('config_file'))

        _presents = [ w for w in p.presence if w.is_present() or w.is_teleworking() if w['name'] not in allowed ]
        _presents = remove(_presents, config.get('config_file'))
        _presents = include(_presents, config.get('config_file'))
        _presents = exclude(_presents, config.get('config_file'))

        for w in _presents:
            if w['name'] not in presents_name:
               presents_list.append(w)
            presents_name.add(w['name'])

        monitor(_presents, dmonitor, config.get('config_file'))

    dmonitor = {k: sorted(v) for k, v in dmonitor.items()}

    log.log("{:10s} {}".format('Presents', stats_str(presents_list, list_total)))

    refined_total = refine(list_total, config.get('config_file'))
    refined_presents = refine(presents_list, config.get('config_file'))

    for option, dv in get_values_from_config(config.get('config_file'), 'refine').items():
        for k in dv:
           t = refined_total[option][k]
           l = refined_presents[option][k]
           log.log("{:10s} {}".format(k, stats_str(l, t)))
           if config.get('verbose', 0) > 1:
               log.log(", ".join([i['name'] for i in l]))

    log.log(", ".join(sorted(presents_name)))

    for key, value in dmonitor.items():
        log.log("{:20s}: {}".format(key, stats_str(value, list_total)))
        if config.get('verbose', 0) > 0:
            log.log(", ".join(value))


def add_parser(parser):

    _parser_presents = parser.add_argument_group('get data options')

    _parser_presents.add_argument('-v', '--verbose',
                                  action='count',
                                  default=0,
                                  help='increase verbosity')

    _parser_presents.add_argument('--csv-prefix',
                                  default=defaults['csv_prefix'],
                                  help='name prefix for csv file. (default %(default)s)')

    _parser_presents.add_argument('--csv-dir',
                                  default=defaults['csv_dir'],
                                  help='directory of csv file. (default %(default)s)')

    _parser_presents.add_argument('--log-file',
                                  default=defaults['log_file'],
                                  help='log file. (default %(default)s)')

    _parser_presents.add_argument('--config-file',
                                  default=defaults['config_file'],
                                  help='configuration file. (default %(default)s)')


def main():

    parser = argparse.ArgumentParser(prog='HR report',
                                     description=__doc__,
                                     formatter_class=argparse.RawTextHelpFormatter)

    add_parser(parser)

    args = parser.parse_args()

    report(**vars(args))


if __name__ == '__main__':
    main()




#!/usr/bin/env python3
import re
import sys
import datetime
import argparse

from .scripts import dump_presents, report, monitor

def add_parser(parser):

    _subparsers = parser.add_subparsers(metavar='')

    # dump presents

    _parser_dump_presents = _subparsers.add_parser('dump-presents',
                                                   help=dump_presents.__doc__)

    dump_presents.add_parser(_parser_dump_presents)

    _parser_dump_presents.set_defaults(todo=dump_presents.dump_presents)

    # report

    _parser_report = _subparsers.add_parser('report',
                                            help=report.__doc__)

    report.add_parser(_parser_report)

    _parser_report.set_defaults(todo=report.report)

    # monitor

    _parser_monitor = _subparsers.add_parser('monitor',
                                             help=monitor.__doc__)

    monitor.add_parser(_parser_monitor)

    _parser_monitor.set_defaults(todo=monitor.monitor_daemon)


def main():

    parser = argparse.ArgumentParser(prog='HRscripts',
                                     description='HR useful scripts.',
                                     formatter_class=argparse.RawTextHelpFormatter)

    add_parser(parser)

    args = parser.parse_args()

    dargs=vars(args)

    for k, v in dargs.items():
        if callable(v):
            v(**dargs)


if __name__ == '__main__':
    main()




#!/usr/bin/env python3
import re
import datetime

from daylog import DayLog
from dayloglist import DayLogList

def main():

   print("\n>>> DayLog tests:")

   print(">>> Null DayLog")

   d = DayLog()
   print(d)

   print(">>> Today DayLog")

   d = DayLog(datetime.datetime.today(), ['09:03', '10:13'])
   print (d)

   print(">>> date()")
   print(d.date())
   print(">>> uptime()")
   print(d.uptime())

   print(">>> logins()")
   print(d.logins())
   print(">>> logouts()")
   print(d.logouts())

   time_start = datetime.time(10, 4)
   time_final = datetime.time(10, 5)

   dstart = datetime.datetime.combine(datetime.date.today(), time_start)
   dfinal = datetime.datetime.combine(datetime.date.today(), time_final)
   print(">>> starting time")
   print(dstart)

   print(">>> uptime from starting time")
   print(d.uptime(dstart))

   print(">>> ending time")
   print(dfinal)
   print(d.uptime(dstart, dfinal))

   d1 = DayLog(datetime.datetime.today(), ['09:03', '10:13'])
   d2 = DayLog(datetime.datetime.today(), ['11:03', '12:13'])

   print(">>> Sum")
   print(">>> d1")
   print(d1)
   print(">>> d2")
   print(d2)

   s = d1+d2

   print(">>> d1 + d2")
   print(s)

   #print(s.logins())


   print("\n>>> DayLogList tests:")

   print(">>> DayLogList null")
   l = DayLogList()
   print(l)

   print(">>> d1")

   d1 = DayLog(datetime.datetime(2014, 12, 3), ['09:03', '10:13'])
   print(d1)

   print(">>> d2")

   d2 = DayLog(datetime.datetime(2014, 12, 4), ['11:03', '15:13'])
   print(d2)

   l.append(d1)
   #print(l)
   l.append(d2)

   print(">>> d1 + d2")
   print(l)


if __name__ == '__main__':
    main()




#!/usr/bin/env python3
import re
import sys
import datetime
import argparse

import dayutils
import daylog

class Logs:

   string_in  = 'IN'
   string_out = 'OUT'

   # private method {{{

   def _get_data_from_files(self):
      data = ''
      for file_log in self._files:
         with open(file_log, "r") as f:
             data += f.read()
      return data

   def _get_extreme_days_from_data(self):
      # get first day of list
      day_id = self._data.split('\n')[ 0].split()[1]
      start = dayutils.str2day(day_id)
      # get last day of list
      day_id = self._data.split('\n')[-2].split()[1]
      end = dayutils.str2day(day_id)
      return start, end


   def _get_logs_from_data(self):
      l = []
      # iterate over days from first to last
      #for d in self._day_start.dayrange(self._day_final):
      for d in dayutils.day_range(self._day_start, self._day_final ):
          logs_day = re.findall('.*'+str(d.strftime(dayutils.fmt_id))+'.*', self._data)
          num_login  = len(re.findall(self.string_in,  " ".join(logs_day)))
          num_logout = len(re.findall(self.string_out, " ".join(logs_day)))

          # if number of login/logout is the same and at least one login
          if num_login and (num_login == num_logout):
              # get only time
              logs_day = [ i.split()[2] for i in logs_day]
              l.append( daylog.DayLog(d, logs_day) )

      # sort logs by decreasing uptime
      l.sort(key = lambda x : x.uptime(), reverse = True)
      return l

   # }}}

   def __init__(self, list_of_file = []):

      # list of log files
      self._files = list_of_file

      # data logs
      self._data = self._get_data_from_files()

      # first and last day
      self._day_start, self._day_final = self._get_extreme_days_from_data()

      # list of day logs (list of DayLog class)
      self._logs = self._get_logs_from_data()


   def __str__(self):
       #def _str(k, v):
       #   return "%s %s\n" % ( k.ljust(30, '.'), v )
       #s = ''
       #s += _str("Read data from file", ", ".join(self._files))
       #s += _str("First day", self._day_start)
       #s += _str("Last day", self._day_final)
       #s += _str("Number of days", self.num_days_total())
       #s += _str("Number of working days", str(self.num_days_work()) + ' (%.0f%%)' % (100.0*self.num_days_work()/self.num_days_total())) 
       #return s
      return self.report()

   def report(self):
      def _str(k, v):
         return "%s %s\n" % ( k.ljust(30, '.'), v )
      s = '\n'
      s += _str("Read data from file", ", ".join(self._files))
      s += _str("First day", self._day_start)
      s += _str("Last day", self._day_final)

      s += _str("Number of days", self.num_days_total())
      s += _str("Number of working days", str(self.num_days_work()) + ' (%.0f%%)' % (100.0*self.num_days_work()/self.num_days_total())) 

      s += _str( "Working time total", self.uptime_total() )
      d = self.uptime_max()
      s += _str( "Working time max", "%s @ %s" % ( str(d.uptime()), str(d.date()) ) )
      d = self.uptime_min()
      s += _str( "Working time min", "%s @ %s" % ( str(d.uptime()), str(d.date()) ) )
      s += _str( "Working time average", self.uptime_mean() )

      # get most prolific week day
      p = self.get_max_uptime_by_day()
      s += _str( "Most prolific week day", dayutils.days_long[ p[1] ] + " (" + str(p[0]) + " working time)")
      # get most prolific month
      p = self.get_max_uptime_by_month()
      s += _str( "Most prolific month", dayutils.months_long[ p[1] ] + " (" + str(p[0]) + " working time)")
      # get most prolific year
      p = self.get_max_uptime_by_year()
      s += _str( "Most prolific year",  str(p[1]) + " (" + str(p[0]) + " working time)")

      return s


   def num_days_total(self):
      """Return total number of days between start and end days"""
      return (self._day_final - self._day_start).days
   def num_days_work(self):
      """Return number of working days between start and end days"""
      return len(self._logs)

   def days_first(self):
      """Return first day"""
      return self._day_start
   def days_last(self):
      """Return last day"""
      return self._day_final


   def uptime_max(self):
      """Return maximum uptime"""
      return self._logs[0]
   def uptime_min(self):
      """Return minimum uptime"""
      return self._logs[-1]

   def uptime_total(self):
      """Return total uptime"""
      up = self.get_from_logs('uptime')
      t = datetime.timedelta(0)
      for i in up:
          t += i
      return t

   def uptime_mean(self):
      """Return mean uptime for day"""
      up = self.uptime_total()
      wd = self.num_days_work()
      return up/wd


   def get_from_logs(self, attribute):
      """Return a list with attribute from daylogs"""
      return [ getattr(i, attribute)() for i in self._logs ]


   def get_logs(self, condition = lambda x : True):
      """Return logs. If condition is specified return a list of logs satisfying the condition"""
      if condition:
         return [ i for i in self._logs if condition(i) ]
         #return sorted([ getattr(i, function)() for i in self._logs if condition(i) ])
      else:
         return self._logs

   def sort_logs(self, attribute, reverse = False):
      """Sort logs by attribute"""
      return sorted( self._logs, key = lambda x : getattr(x, attribute)(), reverse = reverse)

   def get_day(self, day):
      """Return a list of daylogs with specified day of month"""
      return self.get_logs(lambda x : x.day().day == day)
   def get_weekday(self, weekday):
      """Return a list of daylogs with specified day of week"""
      return self.get_logs(lambda x : x.day().weekday() == weekday)
   def get_month(self, month):
      """Return a list of daylogs with specified month"""
      return self.get_logs(lambda x : x.day().month == month)
   def get_year(self, year):
      """Return a list of daylogs with specified year"""
      return self.get_logs( lambda x : x.day().year == year)

   def login_early(self):
      """Return earliest login"""
      return self.sort_logs('login')[0]
   def login_last(self):
      """Return latest login"""
      return self.sort_logs('login', True)[0]

   def logout_early(self):
      """Return earliest logout"""
      return self.sort_logs('logout')[0]
   def logout_last(self):
      """Return latest logout"""
      return self.sort_logs('logout', True)[0]

   def daylog_sum(self, l):
      """Return the sum of a list of daylogs"""
      s = daylog.DayLog()
      for i in l:
          s += i
      return s


   def get_logs_by (self, function, ranges):
      """Return a list (with length of ranges) of a list of daylogs that satisfy : function() == index of ranges"""
      return [ [j for j in getattr(self, function)(i) ] for i in ranges]

   def get_uptime_by_day(self, ranges = range(7)):
      """Return a list of list of daylogs accordling to weekday index.
      Default day ranges [0:7].
      0 is Monday
      [ [Mondays], [Tuesdays], ... , [Sundays] ]"""
      # return a list of list of DayLog with all days devided by week day
      # [ [Mondays], [Tuesdays], ... , [Sundays] ]
      l = self.get_logs_by('get_weekday', ranges)
      # return uptime of DayLogs sum
      return  [ self.daylog_sum(i).uptime() for i in l]

   def get_uptime_by_month(self, ranges = range(1, 13)):
      """Return a list of list of daylogs accordling to month index of ranges.
      Default month ranges [1:13].
      1 is January"""
      # return a list of list of DayLog devided by month
      l =  self.get_logs_by('get_month', ranges)
      # return uptime of DayLogs sum
      return  [ self.daylog_sum(i).uptime() for i in l]

   def get_uptime_by_year(self, ranges = range(0, datetime.datetime.today().year)):
      """Return a list of list of daylogs accordling to year index of ranges.
      Default year ranges [0:taday's year]."""
      # return a list of list of DayLog devided by year
      l =  self.get_logs_by('get_year', ranges)
      # return uptime of DayLogs sum
      return  [ self.daylog_sum(i).uptime() for i in l]



   def get_uptime_by(self, logs_function, search_function):
      """Return uptime and index of uptime accordingly to logs_function and search_function"""
      l = getattr(self, logs_function)()
      uptime = search_function(l)
      index  = l.index(uptime)
      return uptime, index

   def get_max_uptime_by_day(self):
      """Return max uptime and day of max uptime between total uptime of all weekday"""
      return self.get_uptime_by('get_uptime_by_day', max)
   def get_max_uptime_by_month(self):
      """Return max uptime and month of max uptime between total uptime of all month"""
      return self.get_uptime_by('get_uptime_by_month', max)
   def get_max_uptime_by_year(self):
      """Return max uptime and year of max uptime between total uptime of all year"""
      return self.get_uptime_by('get_uptime_by_year', max)


def main():

   parser = argparse.ArgumentParser(prog='',
                                    formatter_class=argparse.ArgumentDefaultsHelpFormatter)

   parser.add_argument('--report',
                       action='store_true',
                       help='print report')

   parser.add_argument('--days-tot',
                       dest='num_days_total',
                       action='store_true',
                       help='number of total days')
   parser.add_argument('--days-work',
                       dest='num_days_work',
                       action='store_true',
                       help='number of working days')
   parser.add_argument('--days-first',
                       dest='days_first',
                       action='store_true',
                       help='first day')
   parser.add_argument('--days-last',
                       dest='days_last',
                       action='store_true',
                       help='last day')


   parser.add_argument('--uptime-tot',
                       dest='uptime_total',
                       action='store_true',
                       help='minimum uptime')
   parser.add_argument('--uptime-max',
                       action='store_true',
                       help='maximum uptime')
   parser.add_argument('--uptime-min',
                       action='store_true',
                       help='minimum uptime')
   parser.add_argument('--uptime-mean',
                       action='store_true',
                       help='mean uptime for working days')


   parser.add_argument('--login-early',
                       action='store_true',
                       help='earlier login')
   parser.add_argument('--login-last',
                       action='store_true',
                       help='latest login')
   parser.add_argument('--logout-early',
                       action='store_true',
                       help='earlier logout')
   parser.add_argument('--logout-last',
                       action='store_true',
                       help='latest logout')

   # Positional parameters (Files)
   parser.add_argument('files',
                       nargs='+',
                       help = "File [File ... ]")

   args = parser.parse_args()

   a = Logs(args.files)

   #print a

   for k, v in sorted(vars(args).items()):
       if v and isinstance(v, bool):
          d = getattr(a, k)()
          if d:
             print( "%s : %s" % (re.sub('_', ' ', k).ljust(15), d) )

   #for i in a.get_logs():
   #   print( i.uptime(), i.date() )

   print( a.report())


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
import re
import sys
import datetime
import argparse

from . import utils
from .logs.daylog import DayLog
from .logs.dayutils import dayutils

def add_parser(parser):

   date_parser = parser.add_argument_group('utils options')

   date_parser.add_argument('--time-interval',
                            nargs='*',
                            metavar='HH:MM',
                            help='calculate time intervals.')

   date_parser.add_argument('--hr2seconds',
                            metavar='HRTIME',
                            help='convert time from HR units to seconds.')

   date_parser.add_argument('--hr2time',
                            metavar='HRTIME',
                            help='convert time from HR units to time.')

   date_parser.add_argument('--time-sum',
                            action=utils.TimeParser,
                            metavar='HH:MM:SS',
                            help='calculate time sum.')

def main():

    parser = argparse.ArgumentParser(prog='HRtime',
                                     description='Time useful utilities.',
                                     formatter_class=argparse.RawTextHelpFormatter)

    add_parser(parser)

    args = parser.parse_args()


    if args.time_interval:

        dl = DayLog(datetime.datetime.today(), args.time_interval)

        print("{} = {}".format("Time Interval", dl.uptime()))


    if args.hr2seconds:

        print("{} = {:.0f}".format("Seconds", utils.hr2seconds(args.hr2seconds)) )


    if args.hr2time:

        print("{} = {}".format("Time [HH:MM:SS]", utils.hr2time(args.hr2time, True)) )


    if args.time_sum:
        time_sum = utils.time_sum(args.time_sum)
        print(dayutils.sec2str(time_sum.total_seconds()))


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
import os
import requests
import getpass
import base64

from configparser import ConfigParser

HRauth_default = {
    'config_file' : os.path.join( os.path.expanduser("~"), '.HRlo'),
    'password_encoding' : 'base64',
    'remove_config_file' : False,
    'save' : False,
    'required' : ['host', 'username', 'idemploy'],
}

class HRauth(dict):

   HRauth_config_option = 'HRauth'

   def __init__(self, **kwargs):

      dict.__init__(self, **HRauth_default)
      # update with arguments
      self.update(kwargs)

      if self['remove_config_file']:
          self._remove_config_file()

      # read from config_file
      self.update( self._read_config_file(self['config_file']) )
      # read from args
      self.update( {k: kwargs[k] for k in self['required'] if kwargs.get(k, None)} )

      self._check_required()

      if not self.get('password', False):
         self['password'] = self._encode(getpass.getpass())

      self._session = requests.Session()

      if self['save']:
          if self.login():
              self._write_config_file(self['config_file'])


   def _encode(self, _pass):

      if self['password_encoding'] == 'base64':
          if isinstance(_pass, bytes):
              _pass = base64.b64encode(_pass)
          else:
             _pass = base64.b64encode(_pass.encode('ascii'))
      else:
          pass

      if isinstance(_pass, bytes):
          _pass = _pass.decode()

      return _pass


   def _decode(self, _pass):

      if self['password_encoding'] == 'base64':
          _pass = base64.b64decode(_pass)
      else:
          pass

      return _pass


   def _check_required(self):
      missing = []
      for k in self['required']:
         if not self.get(k, False):
            missing.append(k)
      if missing:
         raise Exception("[HRauth] *** ERROR *** missing required args: {}".format( ", ".join(missing) ))


   def _read_config_file(self, fname):
      """Read from config file"""

      if not os.path.isfile(fname):
         return {}

      parser = ConfigParser()
      parser.read(fname)

      return {k: parser.get(self.HRauth_config_option, k) for k in parser.options(self.HRauth_config_option)}


   def _write_config_file(self, fname):
       parser = ConfigParser()

       parser.add_section(self.HRauth_config_option)

       if self.get('host', None):
          parser.set(self.HRauth_config_option, 'host', self['host'])
       if self.get('username', None):
          parser.set(self.HRauth_config_option, 'username', self['username'])
       if self.get('idemploy', None):
          parser.set(self.HRauth_config_option, 'idemploy', str(self['idemploy']))

       if self['save_password']:
          parser.set(self.HRauth_config_option, 'password', self['password'])

       with open(fname, "w") as f:
          parser.write(f)


   def _remove_config_file(self):
       if os.path.isfile(self['config_file']):
         print("Removing config file :", self['config_file'])
         os.remove(self['config_file'])


   def host(self):
       return self['host']

   def username(self):
       return self['username']

   def idemploy(self):
       return "{:0>7}".format(str(self['idemploy']))

   def login_url(self):
       return 'https://' + self.host() + '/HRPortal/servlet/cp_login'

   def login(self):
       return self._check_login(self.post())

   def _check_login(self, session):
       try:
          if 'jsp/home.jsp' in session.headers['location']:
             return True
       except:
          raise Exception("[HRauth] *** ERROR *** on HR authentication!")

   def session(self):
       return self._session

   def post(self):
       auth = {'m_cUserName' : self['username'], 'm_cPassword' : self._decode(self['password']), 'm_cAction' : 'login'}
       return self.session().post(self.login_url(), params=auth, allow_redirects=False)



def add_parser(parser):

   authparser = parser.add_argument_group('authentication options')

   authparser.add_argument('-u', '--username',
                           help='Username')

   authparser.add_argument('-i', '--idemploy',
                           help='ID employee')

   authparser.add_argument('-a', '--url',
                           dest='host',
                           help='HR url')

   authparser.add_argument('-c', '--config-file',
                           default=HRauth_default['config_file'],
                           help='Configuration file (default %(default)s)')

   authparser.add_argument('-s', '--save',
                           action='store_true',
                           help='Save HR authentication options in HRauth config file')

   authparser.add_argument('--save-password',
                           action='store_true',
                           help='Save HR authentication password in HRauth config file')

   authparser.add_argument('--remove-config-file',
                           action='store_true',
                           help='Remove HR config file')

   args = parser.parse_args()

   return vars(args)


def main ():

   import argparse

   parser = argparse.ArgumentParser(prog='HRauth',
                                    description='Module to authenticate on HR.',
                                    formatter_class=argparse.RawTextHelpFormatter)
   add_parser(parser)

   args = parser.parse_args()

   auth = HRauth(**vars(args))

   if auth.login():
       print("Successfully login with user: {}".format(auth.username()))


if __name__ == '__main__':
    main()




#!/usr/bin/env python3
import re
import datetime

from .dayutils import dayutils

def pairwise(l):
   iterator = iter(l)
   a = next(iterator)
   for b in iterator:
      yield a, b
      a = next(iterator)
   yield a, b


class DayLog(dict):
   """Class to management of login/logout accesses.
   """

   def __init__ (self, date = None, logs = None):
      """__init__(date, logs).
      """

      # date of day
      self['date'] = date
      # list of logs
      self['logs'] = []
      # total uptime
      self['uptime'] = datetime.timedelta(0)

      if not logs:
         return

      # check if logs are odds
      pass

      self['logs'] = [ datetime.datetime.combine( self['date'], dayutils.str2time(i) ) for i in logs ]

      for i in [ o-i for i, o in pairwise(self['logs']) ]:
         self['uptime'] += i


   def __str__(self):
      s = ''
      if not self['date'] or not self['logs']:
          return s
      s += "[LOGS] "
      s +=          "{:.<20}{}\n".format( 'Date', str(self['date'].date()) )
      s += " "*7 +  "{:.<20}{}\n".format( 'Uptime', dayutils.sec2str(self['uptime'].total_seconds()) )
      s += " "*7 +  "{:.<20}[ {} ]".format( 'Logs', ", ".join([ i.time().strftime(dayutils.fmt_time) for i in self['logs']]) )
      return s

   def __add__(self, other):
      l1 = [i.time().strftime(dayutils.fmt_time) for i in self['logs']]
      l2 = [i.time().strftime(dayutils.fmt_time) for i in other['logs']]
      return DayLog( date = datetime.datetime(1970, 1, 1), logs = l1 + l2 )

   def __radd__(self, other):
      return self.__add__(other)

   def _get_uptime_from(self, start, final):
       """Return uptimes between start and final times."""

       assert isinstance(start, datetime.datetime), \
              "@_get_uptime_from start is not datetime.datetime: %s %s" % \
              (type(start), start)

       assert isinstance(final, datetime.datetime), \
              "@_get_uptime_from final is not datetime.datetime: %s %s" % \
              (type(final), final)

       _time = datetime.timedelta(0)

       for i, o in pairwise(self['logs']):

          if start > o or final < i:
             continue

          i1 = start
          i2 = final

          if start < i:
             i1 = i

          if final > o:
             i2 = o

          _time += i2 - i1

       return _time


   def id (self):
      """Return date in format YYYYMMDD."""
      return self['date'].strftime(dayutils.fmt_id)


   def date (self, fmt=False):
      """Return date.
         If format=True return date in format DD/MM/YYYY."""
      if fmt:
         return self['date'].strftime(dayutils.fmt_day)
      else:
         return self['date']


   def day (self):
      """Return date (datetime class).
         (Deprecated).
      """
      return self['date']


   def uptime(self, start=None, final=datetime.datetime.now()):
      """Return uptime in seconds.
         If start and final is defined return uptime in seconds between start and final times.
      """
      if start:
         return self._get_uptime_from(start, final)
      else:
         return self['uptime']


   def uptimes(self, fmt = None):
       """Return list of uptimes.
          If fmt=True return string with uptimes in format mode.
       """
       ups = [ o-i for i, o in pairwise(self['logs']) ]
       if fmt:
           return [ dayutils.sec2str(u.total_seconds())  for u in ups ]
       else:
           return ups


   def logs (self, fmt = None):
      """Return list of logs.
         If fmt=True return string with logs in format mode.
      """
      if fmt:
         return [ i.strftime(fmt) for i in self['logs']]
      else:
         return self['logs']


   def login (self):
      """Return earlier login."""
      return self['logs'][0]


   def logout (self):
      """Return latest login."""
      return self['logs'][-1]


   def logins (self):
      """Return list of logins."""
      return [i for i, o in pairwise(self['logs']) ]


   def logouts (self):
      """Return list of logouts."""
      return [o for i, o in pairwise(self['logs']) ]





#!/usr/bin/env python3
import re
import argparse
from collections import OrderedDict

from . import HRauth
from . import HRget
from . import utils


class HRphone (OrderedDict):


    def __init__ (self, json):
        super(HRphone, self).__init__()

        self.json = self._json_to_dict(json)


    def _json_to_dict(self, arg_json):
        """Refine data from original json."""

        fields_to_return = ['ANSURNAM', 'ANEMAIL', 'ANTELEF', 'ANMOBILTEL']
        data_to_return   = []

        _json = {}

        for worker in arg_json['Data']:
           j= {k: v for k, v in zip(arg_json['Fields'], worker)}
           l = []
           for k in fields_to_return:
              if j.get(k):
                 l.append(j[k])
           data_to_return.append(l)

        _json['Fields'] = fields_to_return
        _json['Data']   = data_to_return

        return _json


    def get(self, names=[], phones=[]):
        """Filter data with names in names list or phone number in phones list.
           Return json with filtered workers info."""
        re_names = [ re.compile(i.upper()) for i in names ]
        re_phones = [ re.compile(i.upper()) for i in phones ]

        _json = {'Fields': self.json['Fields'], 'Data': self.json['Data']}

        # filter data
        if re_names or re_phones:
           l = []
           for i in self.json['Data']:
               _name  = i[0]
               _phone = " ".join( [ re.sub(r"[^0-9]+", "", j) for j in i[2:] ] )
               l.extend( [ i for r in re_names if r.search(_name) ] )
               l.extend( [ i for r in re_phones if r.search(_phone) ] )
           _json['Data'] = l

        return _json


    def report(self, names=[], phones=[]):
        """Return string report of workers filtered accordingly to names in names list and phone number in phones list.""" 

        djson = self.get(names, phones)

        l = []
        for d in djson['Data']:
            l.append('')
            for k, v in zip(djson['Fields'], d):
                l.append(v)

        if l:
            return '\n'.join(l) + '\n'
        else:
            return ''



def add_parser(parser):

   phone_parser = parser.add_argument_group('phone number options')

   phone_parser.add_argument('-p', '--phone',
                             dest = 'phone_name',
                             default = [],
                             nargs = '+',
                             action = utils.NameParser,
                             metavar = "SURNAME",
                             help="get phone number")

   phone_parser.add_argument('-n', '--name-from-phone',
                             dest = 'phone_number',
                             default = [],
                             nargs = '+',
                             metavar = "PHONE",
                             help="get name from phone number")



def main():

    parser = argparse.ArgumentParser(prog='HRphone',
                                     description='HR manager utility for workers phone number.',
                                     formatter_class=argparse.RawTextHelpFormatter)

    add_parser(parser)

    HRauth.add_parser(parser)

    args = parser.parse_args()


    hr_auth = HRauth.HRauth(**vars(args))

    hr_get = HRget.HRget(hr_auth, verbose=False)

    hr_phone = HRphone(hr_get.phone())


    if args.phone_name or args.phone_number:
        print(hr_phone.report(names = args.phone_name, phones = args.phone_number))



if __name__ == '__main__':
   main()




#!/usr/bin/env python3
from datetime import timedelta, datetime

from .HRday import HRday
from .utils import to_str
from .logs.dayutils import day_range, week_bounds, month_weeks_bounds

class HRdayList(list):


    def __init__(self, label=None, **kwargs):

       self.hrday = HRday()

       self.label = label

       self.config = kwargs

       #self_hr_time = {}
       #self._hr_time['net'] = 0
       #for k in list(HRday.time_hash.keys()) + ['up']:
       #    self._hr_time[k] = timedelta(0)


    def __str__(self):
       s = ''

       if self.label:
          s += "."*25 + "{}\n".format(self.label)

       s += "{:.<25}".format("Working days")
       s += "{}\n".format( ", ".join([ str(i.day().date()) for i in self if i.working()]))
       s += "{:.<25}".format("Num. Working days")
       s += "{}\n".format( self.working() )
       #s += "{:.<25}".format("Working days list")
       #s += "{}\n".format( self.working(list=True) )
       s += "{:.<25}".format( "Uptime" )
       s += "{:<10}".format( to_str(self.hrday.uptime()) )
       s += " {} ".format( "for HR" )
       s += "{}\n".format( to_str(self.hrday['HR times']['up']) )
       s += "{:.<25}".format( "Timenet" )
       s += "{:<10}".format( to_str(self.timenet()) )
       s += " {} ".format( "for HR" )
       s += "{}\n".format( to_str(self.hrday['HR times']['net']) )
       s += "{:.<25}".format( "Time to work" )
       s += "{}\n".format( to_str(self.hrday.get('time_2work', 0)) )
       if self.hrday.get('time_2work'):
           s += "{:.<25}".format( "Worked time in %" )
           s += "{:.1f}%\n".format( 100.0*self.hrday.uptime().total_seconds()/self.hrday.get('time_2work', 1) )
       s += "{:.<25}".format( "Uptime mean" )
       s += "{}\n".format( to_str(self.mean(self.hrday.uptime())) )
       s += "{:.<25}".format( "Timenet mean" )
       s += "{}\n".format( to_str(self.mean(self.hrday.timenet())) )

       s += "{:.<25}".format("Timenets per day")
       s += "[{}]\n".format( ", ".join([ to_str(d.timenet()) for d in self if d.working()]) )

       if len(self.week_bounds()) > 1:
           s += "{:.<25}".format("Timenets per week")
           s += "[{}]\n".format( ", ".join([ to_str(d) for d in self.timenet_weeks() ]) )
           s += "{:.<25}".format("Working days per week")
           s += "{}\n".format( [ len([j for j in i if j]) for i in self._get_list_attr('working', weeks=True) ] )
           s += "{:.<25}".format("Week bounds")
           s += "[{}]\n".format(", ".join( [ "{}/{}".format(s, e) for s, e in self.week_bounds() ] ))

       if self.anomaly():
           s += "{:.<25}".format( "Anomaly" )
           s += "{}\n".format( self.anomaly() )

       s += "{:.<25}".format( "Lunch time" )
       s += "{}\n".format( to_str(self.hrday.get('time_lunch', timedelta(0))) )
       s += "{:.<25}".format( "Lunch time mean" )
       s += "{}\n".format( to_str(self.mean(self.hrday.get('time_lunch', timedelta(0)))) )

       return s


    def append(self, args):
       #if not args.working(): return
       list.append(self, args)

       self.hrday['uptime'] += args.uptime()

       for k in list(HRday.time_hash.keys()) + ['up', 'net']:
          self.hrday['HR times'][k] += args['HR times'][k]

       if not args.anomaly() and not args.holiday():
          self.hrday['timenet'] += args.timenet()

       for k in HRday.times_to_add:
           try:
               self.hrday[k] += args[k]
           except KeyError:
               self.hrday[k] = args[k]


    def __getitem__(self, index):
        if isinstance(index, int):
            return super(HRdayList, self).__getitem__(index)
        else:
            if isinstance(index, datetime):
                index = index.date()
            for d in self:
                if d['date'].date() == index:
                    return d
            raise IndexError("HRdayList index out of range")


    def week_bounds(self):
        _weeks = []
        if not self:
            return _weeks
        d = self[0]['date']
        while (d.date() < week_bounds(self[-1]['date'])[1]):
            _weeks.append(week_bounds(d))
            d += timedelta(days=7)
        return _weeks


    def days_per_week(self):
        _days_per_week = []
        for w in self.week_bounds():
            l = HRdayList()
            for d in day_range(w[0], w[1]):
                try:
                    l.append(self[d])
                except:
                    pass
            _days_per_week.append(l)
        return _days_per_week


    def _get_list_attr(self, attr, days=False, weeks=False):
        """Return list of values of attributes attr from HRday list."""
        if days:
            return [i for i in self if getattr(i, attr)()]
        elif weeks:
            return [ [getattr(d, attr)() for d in w] for w in self.days_per_week() ]
        else:
            return [getattr(i, attr)() for i in self]


    def report(self):
        return str(self)


    def uptime(self, list=False):
        """Return total uptime.
           If list=True return the list of all uptimes."""
        if list:
            return self._get_list_attr('uptime')
        else:
            up = timedelta(0)
            for i in self.uptime(list=True):
                up += i
            return up


    def anomaly(self, list=False):
        """Return number of anomalies.
           If list=True return the list of HRday with anomalies."""
        if list:
            return self._get_list_attr('anomaly', days=True)
        else:
            return len(self.anomaly(list=True))


    def working(self, list=False):
        """Return number of working days.
           If list=True return the list of working HRday."""
        if list:
            return self._get_list_attr('working', days=True)
        else:
            return len(self.working(list=True))


    def timenet(self, list=False):
        if list:
            return self._get_list_attr('timenet')
        else:
            tnet = sum(self.timenet(list=True))
            if self.config.get('overtime'):
                tnet -= timedelta(hours=self.config['overtime']).total_seconds()
            return tnet

    def timenet_weeks(self, list=False):
        if list:
            return self._get_list_attr('timenet', weeks=True)
        else:
            return  [ sum(w) for w in self.timenet_weeks(list=True) ]


    def mean(self, t):
        if self.working() == 0:
            if isinstance(t, timedelta):
                return timedelta(0)
            else:
                return 0
        return t / self.working(list=False)




#!/usr/bin/env python3
import argparse
from collections import OrderedDict

from .utils import HashedDict

from . import HRauth
from . import HRget

class TotalizatorName:

    from_name_to_HRname = {
        # ... HOLIDAYS ............................
        # Holidays Remainders Previous Year
        'HRPY' : 'FERIE - Residuo AP',
        # Holidays be DUE to
        'HDUE' : 'FERIE - Spettante',
        # Holidays Enjoyed Current Year
        'HECY' : 'FERIE - Goduto Annuo',
        # Holidays TO Enjoy
        'HTOE' : 'FERIE - Da Godere',
        # ... ROL .................................
        # ROL Remainders Previous Year
        'RRPY' : 'ROL - Residuo AP',
        # ROL be DUE to
        'RDUE' : 'ROL - Spettante',
        # ROL Enjoyed Current Year
        'RECY' : 'ROL - Goduto Annuo',
        # ROL TO Enjoy
        'RTOE' : 'ROL - Da Godere',
        # ROL Progressive To Liquidate
        'RPTL' : 'ROL - Liquidate Progressive',
        # ... HOURS ...............................
        # Bank Hours Previous Month
        'BHPM' : 'BANCA ORE - Saldo MP',
        # Bank Hours Current Month
        'BHCM' : 'BANCA ORE - Saldo MC',
        # Amount Hours Previous Month
        'AHPM' : 'MONTE ORE - Saldo MP',
        # Amount Hours Current Month
        'AHCM' : 'MONTE ORE - Saldo MC',
    }

    @classmethod
    def name(self, value):
        for k, v in self.from_name_to_HRname.items():
            if v == value:
                return k 

    @classmethod
    def description(self, value):
        return self.from_name_to_HRname[value]

    @classmethod
    def is_name(self, value):
        return value in self.from_name_to_HRname.keys()

    @classmethod
    def is_description(self, value):
        return value in self.from_name_to_HRname.values()

    @classmethod
    def all_names(self):
        return self.from_name_to_HRname.keys()

    @classmethod
    def all_descriptions(self):
        return self.from_name_to_HRname.values()



class Totalizator (HashedDict):

    from_key_to_HRkey = {
        'name'        : 'CONTATORE',
        'value'       : 'QTACONT',
        'description' : 'DESCRIZIONE',
        'id'          : 'ORDINE',
        'unitmis'     : 'UNITAMIS',
        'contzero'    : 'VISCONTZERO',
        'enable'      : 'ABILITATO',
    }

    def __init__(self, *args, **kwargs):
        self.key_table = self.from_key_to_HRkey
        self.update(*args, **kwargs)

    def __call__(self):
        return self.value()

    def value(self):
        return self['value']

    def description(self):
        return self['description']

    def report(self):
        return "{:30s} = {}".format(self['description'], self['value'])

    def is_holiday(self):
        return 'FERIE' in self['description']

    def is_rol(self):
        return 'ROL' in self['description']

    def is_hours(self):
        return 'ORE' in self['description']



class HRtotalizator (OrderedDict):

    def __init__ (self, json):
        super(HRtotalizator, self).__init__()

        for t in json['Data']:
            _totalizator = Totalizator( {k: v for k, v in zip(json['Fields'], t)} )
            name = TotalizatorName().name(_totalizator.description())
            self[name] = _totalizator

    def report(self):
        return "\n".join([i.report() for i in self.values()])

    def get_totalizator(self, key):
        if TotalizatorName().is_name(key):
            return self[key]
        if TotalizatorName().is_description(key):
            return self[TotalizatorName().name(key)]

    def get_value(self, key):
        totalizator = self.get_totalizator(key)
        if totalizator:
            return totalizator.value()

    def dump(self, file_out):
        pass



def add_parser(parser):

   _parser = parser.add_argument_group('totalizator options')

   _parser.add_argument('--tot', '--totalizators',
                        action = 'store_true',
                        dest = 'totalizators',
                        help='report of totalizators')

   _parser.add_argument('--get-totalizator',
                        #choices = TotalizatorName.all_descriptions(),
                        dest = 'get_totalizator',
                        metavar = 'TOTALIZATOR',
                        help='get totalizator value')


def main():

    parser = argparse.ArgumentParser(prog='HRtotalizator',
                                     description='HR manager utility for totalizators.',
                                     formatter_class=argparse.RawTextHelpFormatter)

    add_parser(parser)

    parser.add_argument('--dump',
                        dest = 'file_out',
                        help='dump data to file')

    HRauth.add_parser(parser)

    args = parser.parse_args()

    hr_auth = HRauth.HRauth(**vars(args))

    hr_get = HRget.HRget(hr_auth, verbose=False)

    hr_tot = HRtotalizator(hr_get.totalizators())

    if args.totalizators:
        print(hr_tot.report())

    if args.get_totalizator:
        print(hr_tot.get_value(args.get_totalizator))


if __name__ == '__main__':
   main()




#!/usr/bin/env python3
import datetime
import argparse

from .logs.dayutils import day_range as DayRange
from .logs.dayutils import week_bounds, month_bounds, month_weeks_bounds

from . import HRauth
from . import HRget
from . import HRday
from . import HRdayList
from . import HRphone
from . import HRpresence
from . import HRtotalizator

from . import color
from . import config as HRconfig


class HRlo(object):


   def __init__(self, auth, config = {}, day_range=None):

       self.hr_auth = auth
       self.hr_get = HRget.HRget(self.hr_auth)

       self.days = [] # initialization in init_data

       self.config = {}
       self.config.update( config )


   def __str__ (self):
       s = ''
       for i in self.days:
           s += str(i) + '\n'
       return s


   def __getitem__(self, key):
       if not self.days:
           raise KeyError("no data for {}".format(key))
       if not isinstance(key, datetime.datetime) and \
          not isinstance(key, datetime.date) and \
          not isinstance(key, DayRange) and \
          not isinstance(key, slice):
          print("@__getitem__ NOT datetime.datetime")
          return None
       if isinstance(key, DayRange):
          l = [ d for d in self.days if d.day().date() in key ]
          return l
       elif isinstance(key, datetime.datetime):
          l = [ d for d in self.days if d.day().date() == key.date() ]
          return l
       elif isinstance(key, datetime.date):
          l = [ d for d in self.days if d.day().date() == key ]
          return l
       elif isinstance(key, slice):
          return self.days[key.start.day-1:key.stop.day-1]
       else:
          return self.days[key.day-1]


   def init_data(self, day_range=None):
       if day_range:
          json = self.hr_get.get_range(day_range)
       else:
          json = self.hr_get.get()

       self.days = [ HRday.HRday({'Fields':json['Fields'], 'Data':d}) for d in json['Data'] ]


   def day(self, day = datetime.date.today()):
       """Return HRday class for a day."""
       self.init_data( DayRange(day, day) )
       return self


   def week(self, day = datetime.datetime.today()):
       """Return HRdayList class for a week."""
       start, end = week_bounds(day)
       # check if week start and end are inside current month
       month_limits = month_bounds(day)
       start = max(start, month_limits[0])
       end = min(end, month_limits[1])
       return self.get(start, end, label="Weekly report", overtime=self.config.get('overtime', 0))


   def month(self, day = datetime.datetime.today()):
       """Return HRdayList for a month."""
       start, end = month_bounds(day)
       return self.get(start, end, label="Monthly report")


   def month_weeks(self, day = datetime.datetime.today()):
       """Return a list of HRdayList for month weeks."""
       _hrdaylist = []
       # get month bounds
       month_limits = month_bounds(day)
       # get today bound
       after_bound = datetime.datetime.today().date()
       if not self.config.get('today', True):
           # get yesterday as bound
           after_bound = datetime.datetime.today().date() - datetime.timedelta(days=1)
       for w in month_weeks_bounds(day):
           start = w[0]
           if start > after_bound:
               break
           # check if week end are inside current month
           end = min(w[1], month_limits[1])
           # check if week end are after today
           end = min(end, after_bound)
           # get HRdayList for this week
           _hrdaylist.append(self.get(start, end, overtime=self.config.get('overtime', 0)))
       return _hrdaylist


   def get(self, start, end, label='', overtime=0):
       """Return HRdayList for a day interval."""
       day_range = DayRange(start, end)

       self.init_data(day_range)

       if start == end:
           _label = str(start)
       else:
           _label = "From {} To {}".format( str(start), str(end) )

       if label:
           _label = "{} : {}".format( label, _label)

       l = HRdayList.HRdayList(label=_label, overtime=overtime)
       for i in self[day_range]:
           if i.is_today() and not self.config.get('today', False): continue
           l.append(i)

       return l


   def report(self, start, end):
       return str(self.get(start, end))


   def report_day(self, day = datetime.date.today()):
       """Return report from HRday class for a day."""
       return str(self.day(day))


   def report_week(self, day = datetime.datetime.today()):
       """Return report from HRdayList class for a week."""
       return str(self.week(day))


   def report_month(self, day = datetime.date.today()):
       """Return report from HRdayList class for a month."""
       return str(self.month(day))


   def report_month_weeks(self, day = datetime.date.today()):
       """Return report from HRdayList class for a month."""
       return "\n".join([ str(w) for w in self.month_weeks(day)])


   def report_keys(self, keys = [], from_day = datetime.date(datetime.date.today().year, 1, 1), to_day = datetime.date.today()):
       """Return keys report from HR time of HRdayList class for day range."""
       l = self.get(from_day, to_day)
       t = datetime.timedelta(0)
       days = []
       for d in l:
           for k in keys:
               _time = d._get_hr_time(k)
               t += _time
               if _time:
                   days.append(d)
       if self.config.get('verbose'):
           if days:
               print("number of days =", len(days))
               print("days =", ", ".join([str(d['date'].date()) for d in days]))
       return t


   def anomalies(self):
       _anomalies = [d for d in self.days if d.anomaly()]
       if _anomalies:
           col = color.color(color.RED)( str )
           warning = "WARNING : {} anomalies found : {}".format(len(_anomalies),  [ str(d.day().date()) for d in _anomalies ] )
           print( col(warning) )
       return _anomalies


   def phone(self, names=[], phones=[]):
       """Return report of names and phone numbers from HRphone class."""
       djson = self.hr_get.phone()
       hr_phone = HRphone.HRphone(djson)
       return hr_phone.report(names, phones)


   def presence(self, surname):
       """Return report of workers presence from HRpresence class."""
       csv_data = self.hr_get.presence()
       hr_presence = HRpresence.HRpresence(csv_data)
       return hr_presence.report(surname)


   def totalizator(self, key=None):
       """Return report of totalizators from HRtotalizator class."""
       hr_totalizator = HRtotalizator.HRtotalizator(self.hr_get.totalizators())
       if key:
           return hr_totalizator.get_value(key)
       else:
           return hr_totalizator.report()



def main():

   parser = argparse.ArgumentParser(prog='HRlo (aka accaerralo)',
                                    description='HR manager.',
                                    formatter_class=argparse.RawTextHelpFormatter)

   parser.add_argument('--version', action='version',
                       version='%(prog)s ' + HRconfig.version,
                       help='print version and exit')

   parser.add_argument('-v', '--verbose',
                       action="count", default=0,
                       help="increase verbosity")

   parser_todo = parser.add_argument_group('report options')

   parser_todo.add_argument('-d', '--daily',
                            action='store_true',
                            help='daily report')

   parser_todo.add_argument('-w', '--weekly',
                            action='store_true',
                            help='weekly report')

   parser_todo.add_argument('-m', '--monthly',
                            action='store_true',
                            help='monthly report')

   parser_todo.add_argument('-M', '--week-monthly',
                            action='store_true',
                            help='monthly report week by week')

   parser_todo.add_argument('-t', '--today',
                            action='store_true',
                            help='keep today in reports')

   parser_todo.add_argument('-o', '--overtime',
                            type=int,
                            default=0,
                            help='overtime hours. Remove overtime from timenets. Only supported in weekly and month week by week report. (default %(default)s)')

   parser_todo.add_argument('--report-keys',
                            nargs='+',
                            help='report on HR keys. By default report starts from first day of current year until today. Otherwise use --from and --to options to define time range')


   parser_range = parser.add_argument_group('range days options')

   def _date(s):
      for fmt in ['%Y-%m-%d', '%Y%m%d', '%d-%m-%Y', '%d/%m/%Y']:
         try:
            return datetime.datetime.strptime(s, fmt).date()
         except:
            pass
      raise ValueError("invalid date {}".format(s))

   parser_range.add_argument("--from",
                             dest = 'from_day',
                             type=_date,
                             default=None, #default=datetime.datetime(1970, 1, 1),
                             metavar="YYYY-MM-DD",
                             help="from date YYYY-MM-DD")

   parser_range.add_argument("--to",
                             dest = 'to_day',
                             type=_date,
                             default=datetime.date.today(),
                             metavar="YYYY-MM-DD",
                             help="to date YYYY-MM-DD (default %(default)s)")

   HRpresence.add_parser(parser)

   HRphone.add_parser(parser)

   HRtotalizator.add_parser(parser)

   HRauth.add_parser(parser)

   args = parser.parse_args()


   config = {'today' : args.today, 'overtime' : args.overtime, 'verbose' : args.verbose}

   hr_auth = HRauth.HRauth(**vars(args))

   hr = HRlo(hr_auth, config)


   if args.from_day and args.to_day and not args.report_keys:
       print(hr.report(args.from_day, args.to_day))

   if args.daily:
       print(hr.report_day())

   if args.weekly:
       print(hr.report_week())

   if args.monthly:
       print(hr.report_month())

   if args.week_monthly:
       print(hr.report_month_weeks())

   if args.phone_name or args.phone_number:
       print(hr.phone(names = args.phone_name, phones = args.phone_number))

   if args.presence:
       print(hr.presence(args.presence))

   if args.totalizators or args.get_totalizator:
       print(hr.totalizator(args.get_totalizator))

   if args.report_keys:
       if args.from_day and args.to_day:
           print(hr.report_keys(args.report_keys, args.from_day, args.to_day))
       elif not args.from_day and args.to_day:
           print(hr.report_keys(args.report_keys, to_day=args.to_day))
       else:
           print(hr.report_keys(args.report_keys))

   if not args.daily and not args.weekly and not args.monthly and not args.week_monthly \
      and not args.from_day \
      and not args.phone_name and not args.phone_number and not args.presence \
      and not args.totalizators and not args.get_totalizator and not args.report_keys:
      today = hr.report_day()
      if today:
          print("\nToday :")
          print(today)


if __name__ == '__main__':
   main()




#!/usr/bin/env python3

name    = 'HRlo'
version = '0.7.14'

description = 'HR manager'
long_description = 'HR manager'

url = 'https://github.com/so07/HRlo'

author='so07'
author_email=''

maintainer=author



#!/usr/bin/env python3
import logging
import datetime
import random

class logit(object):


    def __init__(self, name="", **kwargs):

        if kwargs.get('log_file'):
            self.logger = self.get_logger(name, **kwargs)
        else:
            self.logger = None


    def get_logger(self, name="", datefmt="%Y-%m-%d/%H:%M:%S", **config):
    
        time_format = datetime.datetime.now().strftime("%Y-%m-%d-%H:%M:%S")

        # logger
        logger = logging.getLogger(name+time_format+str(random.random()))
        logger.setLevel(logging.INFO)
        # create file handler which logs even debug messages
        fh = logging.FileHandler(config['log_file'])
        # create formatter and add it to the handlers
        formatter = logging.Formatter('%(asctime)s %(message)s', datefmt=datefmt)
        fh.setFormatter(formatter)
        # add the handlers to the logger
        logger.addHandler(fh)
    
        return logger


    def log(self, msg=""):

        if self.logger:
            self.logger.info(msg)
        else:
            print(msg)




#!/usr/bin/env python3
import re
import csv
import json
import argparse
import itertools

from .utils import HashedDict, NameParser

from . import HRauth
from . import HRget

class presence (HashedDict):

    from_key_to_HRkey = {
        'date'           : 'Date()'                                            , 
        'name'           : 'AAA_ElencoPresenti.c_Cognome_Nome'                 , 
        'office'         : 'AAA_ElencoPresenti.c_Sede'                         , 
        'room'           : 'AAA_ElencoPresenti.c_Stanza'                       , 
        'boss'           : 'AAA_ElencoPresenti.c_Resp_Struttura_Organizzativa' , 
        'status'         : 'AAA_ElencoPresenti.c_Presente'                     , 
        'state'          : 'AAA_ElencoPresenti.c_IdStateOggi'                  , 
        'proof'          : 'AAA_ElencoPresenti.c_GiustOggi'                    , 
        'city'           : 'AAA_ElencoPresenti.c_IdProvOggi'                   , 
        'email'          : 'AAA_ElencoPresenti.c_E_mail'                       , 
        'phone'          : 'AAA_ElencoPresenti.c_Tel_Interno'                  , 
        'company phone'  : 'AAA_ElencoPresenti.c_Cell_Aziendale'               , 
        'department'     : 'AAA_ElencoPresenti.c_Struttura_Organizzativa'      , 
        'state tomorrow' : 'AAA_ElencoPresenti.c_IdStateDom'                   , 
        'proof tomorrow' : 'AAA_ElencoPresenti.c_GiustDom'                     , 
        'city tomorrow'  : 'AAA_ElencoPresenti.c_IdProvDom'                    , 
    }

    def __init__(self, *args, **kwargs):
        self.key_table = self.from_key_to_HRkey
        self.update(*args, **kwargs)

    def report (self):
        s  = "{} : {} {}\n".format(self['name'], self['status'], self['proof'])
        s += "{}\n".format(self['email'])
        s += "{}  {}\n".format(self['phone'], self['company phone'])
        s = """{} : {}  {} {} {}
        {}
        {} {}
        {}  @[{}]
        {}  ({})""".format(
            self['name'],
            self['status'], self['proof'], self['city'], self['state'],
            self['email'],
            self['phone'], self['company phone'],
            self['office'], self['room'],
            self['department'], self['boss'],
            )
        if self['proof tomorrow'] or self['city tomorrow'] or self['state tomorrow']:
            s += """
        TOMORROW: {} {} {}""".format(self['proof tomorrow'], self['city tomorrow'], self['state tomorrow'])
        return s

    def is_like (self, key, value):
        m = re.search(value.upper(), self[key].upper())
        if m:
            return True
        else:
            return False

    def is_present(self):
        return 'PRESENTE' in self['status']

    def is_teleworking(self):
        return 'TELELAVORO' in self['proof']

    def is_at(self, city):
        pass


class HRpresence (object):

    def __init__ (self, json):
        self.csv_data = json['csv_data']
        self.raw_presence = [ presence(r) for r in self._csv_read(self.csv_data) ]
        self.presence = self._csv_refine()

    def _csv_refine(self):

        name_unique = set([ i['name'] for i in self.raw_presence ])

        _workers = []
        _refined = []

        # loop on raw data
        for w in self.raw_presence:
            if w['name'] not in _workers:
                _workers.append(w['name'])
                _refined.append(w)

        return _refined
        
    def _csv_read(self, csv_data):
        # convert csv data from string to list
        list_csv_data = csv_data.split('\n')
        #reader = csv.reader(list_csv_data, delimiter=';')
        reader = csv.DictReader(list_csv_data, delimiter=';')
        return reader

    def dump_csv(self, f):
        with open(f, 'w') as fp:
            fp.write(self.csv_data)

    @classmethod
    def read_csv(self, f):
        with open(f, 'r') as fp:
            csv_data = fp.read()
        return {'csv_data' : csv_data}

    def dump_json(self, f):
        with open(f, 'w') as fp:
            json.dump({'csv_data' : self.csv_data}, fp)

    @classmethod
    def read_json(self, f):
        with open(f, 'r') as fp:
            j = json.load(fp)
        return j

    def __len__(self):
        return len(self.presence)
        return len(self.raw_presence)

    def get (self, key, value='', raw=False):
        data = self.presence
        if raw:
            data = self.raw_presence
        return [ i for i in data if i.is_like(key, value) ]

    def get_all (self, key, value=''):
        s = set( [ i[key] for i in self.get(key, value) ] )
        return sorted(list(s))

    def get_zip_keys (self, keys):
        r = [ [] ] * len(self)
        for k in keys:
            l = [ [ i[k] ] for i in self.get(k) ]
            r = [ l2+l1  for l1, l2 in itertools.zip_longest(l, r) ]
        # return unique lists in list
        return list(k for k,_ in itertools.groupby(sorted(r)))

    def get_zip (self, keys, values=[]):
        r = []
        zip_keys = self.get_zip_keys(keys)
        if not values or not [ i for i in values if i ]:
            return zip_keys
        for value_from_worker in zip_keys:
            add_value = True
            counter_keys = 0
            for k, value_from_args in itertools.zip_longest(keys, values):
                if not value_from_args:
                    value_from_args = ''
                if not value_from_args.upper() in value_from_worker[counter_keys].upper():
                    add_value = False
                counter_keys += 1
            if add_value:
                r.append(value_from_worker)
        # return unique lists in list
        return list(k for k,_ in itertools.groupby(sorted(r)))

    def report(self, name):

        if isinstance(name, str):
            workers = self.get('name', name)
            l = [w.report() for w in workers]
            s = '\n'.join(l)
            s = '\n' + s + '\n'

        elif isinstance(name, list):
            l = [self.report(i) for i in name]
            s = ''.join(l)

        return s



def add_parser(parser):

   _parser = parser.add_argument_group('presence options')

   _parser.add_argument('--in', '--is_present',
                        dest = 'presence',
                        nargs='+',
                        action = NameParser,
                        metavar = 'SURNAME',
                        help='get report on worker')



def main():

    parser = argparse.ArgumentParser(prog='HRpresence',
                                     description='HR manager utility for workers presence.',
                                     formatter_class=argparse.RawTextHelpFormatter)

    add_parser(parser)

    parser.add_argument('--dump',
                        dest = 'file_out',
                        help='dump data to file')

    parser.add_argument('--read',
                        dest='file_input',
                        help='read data from file')

    parser.add_argument('--format',
                        dest='file_format',
                        choices=['csv', 'json'],
                        default='csv',
                        help='file format')

    HRauth.add_parser(parser)

    args = parser.parse_args()


    if args.file_input and args.file_format == 'csv':

        djson = HRpresence.read_csv(args.file_input)

    elif args.file_input and args.file_format == 'json':

        djson = HRpresence.read_json(args.file_input)

    else:

        hr_auth = HRauth.HRauth(**vars(args))
        hr_get = HRget.HRget(hr_auth, verbose=False)

        djson = hr_get.presence()


    p = HRpresence(djson)

    if args.presence:
        for name in args.presence:
            print(p.report(name))

    if args.file_out and args.file_format == 'csv':
        p.dump_csv(args.file_out)
    elif args.file_out and args.file_format == 'json':
        p.dump_json(args.file_out)


if __name__ == '__main__':
   main()

